@@ -1,8 +1,9 @@
 import logging
+import os
 from typing import Iterable, List
 
 import opentelemetry.trace
-from opentelemetry.exporter.otlp.proto.grpc import trace_exporter
+from opentelemetry.exporter.otlp.proto.http import trace_exporter
 from opentelemetry.sdk.resources import Attributes, Resource
 from opentelemetry.sdk.trace import SpanProcessor, TracerProvider
 from opentelemetry.sdk.trace.export import (
@@ -10,6 +11,8 @@ from opentelemetry.sdk.trace.export import (
     ConsoleSpanExporter,
     SimpleSpanProcessor,
 )
+
+# noinspection PyProtectedMember
 from opentelemetry.util._once import Once
 
 from troncos.traces.dd_exporter import OTLPSpanExporterDD
@@ -20,6 +23,16 @@ _GLOBAL_SPAN_PROCESSORS_SET_ONCE = Once()
 _DEBUG_SPAN_PROCESSOR: SpanProcessor = SimpleSpanProcessor(ConsoleSpanExporter())
 
 
+def http_endpoint_from_env(host_var: str, port_var: str, path: str = "") -> str | None:
+    host = os.environ.get(host_var)
+    if not host:
+        return None
+    port = os.environ.get(port_var)
+    if not port:
+        return None
+    return f"http://{host}:{port}{path}"
+
+
 def _set_span_processors(span_processors: list[SpanProcessor]) -> None:
     def set_sp() -> None:
         global _GLOBAL_SPAN_PROCESSORS
@@ -70,7 +83,7 @@ def init_tracing_provider(
     provider the global one. If that is not desired, pass in global_provider=False.
     """
 
-    if not _GLOBAL_SPAN_PROCESSORS:
+    if _GLOBAL_SPAN_PROCESSORS is None:
         raise RuntimeError("Call 'init_tracing_endpoint' before calling this function")
 
     if not attributes.get("service.name"):
